% Simulate an indirect MRAC
% for a motor k/(1+sT)

% True system
noiselevel = 0*0.01;
lambda=1;
k = 1;   
T = 5; 
s=tf('s');
G=k/(s*(1+s*T));
h = 1;     % sample period
H=c2d(G,h);
[btrue,atrue] = tfdata(H,'v');
btrue=btrue(2:end);
N = 1000;   % nr of samples
btrue = btrue'*([ones(1,N/2) 1.5*ones(1,N/2)]); 
atrue = atrue'*ones(1,N);
thetatrue = [atrue(2,:);atrue(3,:);btrue(1,:);btrue(2,:)];

% Simulate adaptive controller

time=(1:N)';
wc = 0.05; % square wave reference signal
uc = 1*sign(sin(wc*time));
uc = randn(size(time)); % better excitation
%uc = 0.5*sin(wc*time) + 0.7*sin(2*wc*time);
%Some initial guesses
b=[0.1 0.1];
a=[1 -1.8 0.8];
bplus = 1;
bminus = b;
bm1=1;
am=poly([0.8 0.8]);
ao=[1 -0.6];
[r,s,t]=rstd(bplus,bminus,a,bm1,am,ao);

% initialise vectors
u=zeros(N,1);
y=zeros(N,1);
theta=zeros(4,N);
theta(:,1)=[a(2);a(3);b(1);b(2)];
theta(:,2)=[a(2);a(3);b(1);b(2)];

phi = zeros(4,N);
K=zeros(4,N);
e=zeros(N,1);
Pinv=zeros(4,4,N);
Pinv(:,:,2)=1e-4*eye(4);

for k=3:N  
  n = noiselevel*randn;
  y(k) = -atrue(2:end,k)'*y(k-1:-1:k-2) + btrue(:,k)'*u(k-1:-1:k-2)+n;
  
  phi(:,k) = [-y(k-1); -y(k-2); u(k-1); u(k-2)];
  Pinv(:,:,k) = lambda*Pinv(:,:,k-1) + phi(:,k)*phi(:,k)';
  K(:,k) = Pinv(:,:,k)\phi(:,k);
  e(k) = y(k)-phi(:,k)'*theta(:,k-1);
  theta(:,k) = theta(:,k-1) + K(:,k)*e(k); 
  
  a=[1 theta(1,k) theta(2,k)]; b=[theta(3,k) theta(4,k)];
  [r,s,t]= rstd(1,b,a,1,am,ao);
  u(k) = -r(2)*u(k-1) - s*y(k:-1:k-1) + t*uc(k:-1:k-1);     
end

figure(1)
title('parameter estimates')
subplot(221)
plot(theta(1,:));
hold on;
plot(thetatrue(1,:),'r')
subplot(222)
plot(theta(2,:));
hold on;
plot(thetatrue(2,:),'r')
subplot(223)
plot(theta(3,:));
hold on;
plot(thetatrue(3,:),'r')
subplot(224)
plot(theta(4,:));
hold on;
plot(thetatrue(4,:),'r')


figure(2)
subplot(211)
title('adaptive control of motor')
plot(y);grid on; hold on
plot(uc,'k')
subplot(212)
plot(u);grid on; hold on


